################################################################################
### WAT-E1100                                                                ###
### STATISTICAL ANALYSIS OF GLOBAL DEMOGRAPHIC DATA WITH R                   ###
################################################################################

# Original script 2016 by Timo Räsänen.
# Revised by Joseph Guillaume, Matias Heino, and Vili Virkki.

# INTRODUCTION -----------------------------------------------------------------

# This exercise has five parts (+ reporting):
# 1. Reading data into R
# 2. Plotting data in R
# 3. Correlation analysis
# 4. Regression analysis
# 5. Descriptive analysis of variances 

# Follow the provided code and the exercise instructions. In parts of the
# exercise, some code is missing; you will need to fill these parts in. In the
# end, provide a report which answers to the questions given in the exercise
# instructions.

# DATA USED --------------------------------------------------------------------

# Population [thousands].
# Life expectancy [years].
# Under five year mortality [deaths under age five per 1,000 live births].
# Gross domestic product (GDP) per capita [USD].
# Human development index (HDI) [-].

# All data from United Nations.

# HELP WITH WRITING R SCRIPTS --------------------------------------------------

# Help is available in the lower right window (e.g. search for a function by
# name). Writing ? followed by a function name in the command line also opens
# the help page (e.g. ?mean). R has a strong community around it; most likely,
# someone else has had similar issues than you are having and has asked about it
# in some forums. Hence, googling typically helps in finding example problems
# and solutions.

# RUNNING THE R SCRIPTS --------------------------------------------------------

# One line: Place the cursor on the line you want to run, and press 'Run' in the
# upper right corner of this window or press Ctrl+Enter
# Several lines: Paint the lines you want to run and press 'Run' in the upper
# right corner of this window or press Ctrl+Enter

# EXPORTING PLOTS FROM RSTUDIO -------------------------------------------------

# The quickest way to export plots is directly from the plot window: Lower right
# window -> Plots -> Export -> Save as image/Save as PDF

# GETTING STARTED --------------------------------------------------------------
# 1. Create a new R project from Version Control using Git
#    (File -> New project -> Version Control -> Git)
#    Repository URL: https://version.aalto.fi/gitlab/wdrg/WAT-E1100
# 2. Open R script: WAT-E1100_statistical_analysis_script.R


################################################################################
# Part 1: Reading data into R --------------------------------------------------
#   Read population, life expectancy and under five-year mortality data from
#   csv files into R data frames

# clear any previous variables in the workspace
rm(list=ls())

# read the first few lines to see what the data looks like
# relative path - working directory should be set to the folder with this script
readLines("Population_UN.csv", n = 5)

# use function read.csv to read population data into a variable (see ?read.csv)
# we provide the following arguments for read.csv:
#   file = path to file with data; first parameter, not named
#   header = true/false whether the file contains headers
#   sep = data separator character
#   dec = decimal separator character
population <- read.csv("Population_UN.csv", header = TRUE, sep = ";", dec = ".")

# examine the population data
population # just print it all into the console
head(population) # get the headers and the first rows
str(population) # summary of data columns
summary(population) # numeric summary of values
class(population) # object class
View(population) # open into a new tab - can also click from Environment panel

# TODO: read the life expectancy and under five-year mortality data from
# csv files into data frames (don't use the all_countries files yet, though)

life_expectancy <- NA # REPLACE NA WITH YOUR OWN CODE HERE 
under_five_year_mortality <- NA # REPLACE NA WITH YOUR OWN CODE HERE

# TODO: using the functions introduced above, check the two data sets you just
# read in to confirm that those were read correctly


################################################################################
# Part 2: Plotting -------------------------------------------------------------
#   Plot population, life expectancy and under five-year mortality data using
#   a package called ggplot2

# load ggplot2 package
library(ggplot2)

# plot population data
# function aes(...) specifies an aesthetic mapping describing how visual
#   properties (aesthetics) relate to variables/data values
# any geom_xxx(...) function specifies the geometric object to be used,
#   here lines (e.g. geom_point(...) or geom_col(...) could be used as well)

ggplot(data = population) +
  geom_line(aes(x = Year, y = World,
                color = "World")) +
  geom_line(aes(x = Year, y = High_income_countries,
                color = "High_income_countries")) +
  geom_line(aes(x = Year, y = Middle_income_countries,
                color = "Middle_income_countries")) +
  geom_line(aes(x = Year, y = Low_income_countries,
                color = "Low_income_countries")) +
  ylab("Population [thousands]") +
  labs(title = "Population") +
  scale_x_continuous(breaks = seq(1950, 2015, 10))

# TODO: following the example above, plot life expectancy for years 1960-2014
# for the global values and for the country groups. Limit the x axis values to
# cover only the period with data.

ggplot(data = life_expectancy) + NA
  # REPLACE NA WITH YOUR OWN CODE HERE

# TODO: following the example above, plot under five-year mortality for time
# periods 1950 to 2015 for the global values and for the country groups. Use
# geom_point(...) to draw point geometries.

ggplot(data = under_five_year_mortality) + NA
  # REPLACE NA WITH YOUR OWN CODE HERE


################################################################################
# Part 3: Correlation analysis -------------------------------------------------
#   Examine the relationship of between gross domestic product (GDP) and life
#   expectancy and under five-year mortality. Use Pearson and Kendall methods
#   for determining correlation coefficients.

# read data given in country level
under_five_mort_countries <- read.csv("Under_five_year_mortality_all_countries_UN.csv",
                                      header = TRUE, sep = ";",dec = ".")
life_exp_countries <- read.csv("Life_expectancy_all_countries_UN.csv",
                               header = TRUE, sep = ";",dec = ".")
gdp_countries <- read.csv("GDP_all_countries_UN.csv",
                          header = TRUE, sep = ";",dec = ".")

# GDP and life expectancy - visual inspection ----------------------------------

# for a quick look, plot the relationship between variables using a scatter plot
# created by base-R plot function
plot(gdp_countries$X2011, life_exp_countries$X2011, # see "dollar notation"
     main = "Life expectancy vs. GDP",
     ylab = "Life expectancy [years]", xlab="GDP [USD per capita]")

# Partial check of bivariate normality assumption for Pearson p-value. If either
# of the variables is not normally distributed, the p-value may be unreliable
# for small samples.
hist(gdp_countries$X2011,
     main = "Gross Domestic Propduct",
     xlab = "GDP [USD per capita]")

hist(life_exp_countries$X2011,
     main = "Life expectancy",
     xlab="Life expectancy [years]")

# Quantile-quantile plot shows how theoretical quantiles of normal distribution
# compare to sample quantiles. If the theoretical and sample quantiles matched,
# they would fall on a straight line.
qqnorm(gdp_countries$X2011,
       main = "GDP - Normal probability plot")

qqnorm(life_exp_countries$X2011,
       main = "Life expectancy - Normal probability plot")

# GDP and life expectancy - statistical tests for correlation ------------------

# Shapiro-Wilk's test to analyse normality.
#   H0: data is of normal distribution
shapiro.test(gdp_countries$X2011)
shapiro.test(life_exp_countries$X2011)

# Pearson correlation coefficient - test for linear association; may be affected
# by outliers and high levels of skewness.
#   H0: variables are statistically independent (r = 0)
cor.test(gdp_countries$X2011, life_exp_countries$X2011, method = "pearson")

# Kendall correlation coefficient - test for monotonic association
#   H0: variables are statistically independent (tau = 0)
cor.test(gdp_countries$X2011, life_exp_countries$X2011, method = "kendall")

# GDP and under five-year mortality - determine correlation --------------------

# TODO: determine whether GDP and under five-year mortality correlate with each
# other. Use both Pearson and Kendall correlation coefficients.

# WRITE YOUR OWN CODE HERE


################################################################################
# Part 4: Regression analysis --------------------------------------------------
#   Examine further the relationship between GDP and life expectancy using
#   linear regression; how much GDP explains the variation in life expectancy
#   between countries?

# load lmtest library; required for some if tge regression diagnostics
library(lmtest)

# Data preparation -------------------------------------------------------------

# define data for linear regression (y = a + bx)
y <- life_exp_countries$X2011 # dependent variable
x <- gdp_countries$X2011 # independent variable

# quick-check with scatter plot
plot(x, y, main = "Life expectancy vs. GDP",
     ylab = "Life expectancy [years]", xlab = "GDP [USD per capita]")

# check normality of variables
hist(y, xlab = "Life expectancy [years]",
     main = "Life expectancy (y)")
shapiro.test(y)

hist(x, xlab = "GDP [USD per capita]",
     main = "Gross Domestic Product (GDP) (x)")
shapiro.test(x)

# transform x with log transform to make the relationship more linear
# for other transformations, see for example:
# http://stattrek.com/regression/linear-transformation.aspx?Tutorial=AP
x_log10 <- log10(x)

# check normality again for the transformed x
hist(x_log10, xlab = "logarithmic GDP [log10(USD/cap)]",
     main = "Gross Domestic Product (GDP) (x)")
shapiro.test(x_log10)

# Fit and examine linear regression model --------------------------------------

# fit linear model to data using lm function and a formula denoting dependence
lr_model <- lm(y ~ x_log10)

# examine the estimated model
summary(lr_model)

# T-test for slope parameter(s)
#   H0: slope = 0 (i.e. no linear correlation)
# F-test for regression significance
#   H0: the fit of an intercept-only model and your model are equal

# You can also conclude that if H0 is rejected (p < alpha), the R^2 is different
# from zero (i.e. there is statistically siginificant relationship between the
# variables).

# Regression diagnostics -------------------------------------------------------

# Assumptions for an adequate regression model:
# 1. The relationship between y and x is linear.
# 2. The residuals are independent. In other words, there is no autocorrelation
#    in residuals.
# 3. The residuals follow normal distribution
# 4. The residuals have equal/constant variance, in other words, residuals are
#    homoscedastic/not heteroscedastic .

# check visually the relationship between y and log transformed x
plot(x_log10, y, main= "Relationship between y and log10(x)",
     ylab = "Life expectancy [years]", xlab="logarithmic GDP [log10(USD/cap)]")

# Harvey-Collier test for regression linearity (assumption 1)
#   H0: regression is linear
harvtest(y ~ x_log10)

# get residuals (observed value - predicted value) of the model
res <- resid(lr_model)

# plot autocorrelation function (acf) to detect autocorrelation with 95%
# confidence level (see ?acf for explaining the lag zero autocorrelation = 1)
acf(res)

# Breusch-Godfrey test: independence of model residuals, i.e. lack of
# autocorrelation (assumption 2)
#   H0: autocorrelation of the data is 0
bgtest(lr_model)

# standardise residuals to mean = 0 & standard deviation = 1
stdres <- (res - mean(res)) / sd(res)

# plot a histogram of standardised residuals
hist(stdres, main = "Standardised residuals", xlab = "Normal scores")

# plot a quantile-quantile plot of standardised residuals
qqnorm(stdres, main = "Normal probability plot",
       ylab = "Standardised residuals", xlab = "Normal scores")
qqline(stdres, col = "red") # add a line to the plot

# Shapiro-Wilk's normality test for the residuals (assumption 3)
#   H0: data is of normal distribution
shapiro.test(stdres)

# get predicted y values and standardise
predict_y <- predict(lr_model)
stdpredict <- (predict_y - mean(predict_y)) / sd(predict_y)

# standardized predicted values vs.standardized residuals plot; if points are
# dispersed around the zero line, data is homoscedastic (i.e. variance is
# constant throughout the data)
plot(stdpredict, stdres, main = "Testing heteroscedasticity",
     xlab = "Standardized predicted Values", ylab = "Standardized residuals")
abline(0, 0, col = "red") 

# Breusch-Pagan test for residual heteroscedasticity (assumption 4)
#   H0: data is homoscedastic 
bptest(lr_model)

# Plot data with the estimated regression line ---------------------------------

plot(x_log10, y,
     main = "Life expectancy vs. logarithmic GDP",
     ylab = "Life expectancy [years]",
     xlab = "logarithmic GDP [log10(USD/cap)]")

ab <- lr_model$coefficients # get estimated model parameters
abline(ab[1], ab[2], col = "red") # draw a line representing the model

# NOTE: It is common that all model assumptions are not initially satisfied, and
# there are ways and methods to satisfy the  model assumptions. For example,
# data can be treated and transformed and the regression model can be modified.
# However, these measures are beyond the scope of this exercise.


################################################################################
# Part 5: Variance analysis ----------------------------------------------------
#   Development of global equality in GDP, Human Development Index (HDI), life
#   expectancy, and under five-year mortality. Use variablility between
#   countries as an indicator of equality.

# read one more data: Human Development Index (HDI)
hdi_countries <- read.csv("HDI_all_countries_UN.csv", header = TRUE, sep = ";",
                          dec = ".")

# Calculate and plot variance --------------------------------------------------

# drop the first column (country names) by selecting all except the first column
mort_values <- under_five_mort_countries[,2:ncol(under_five_mort_countries)]

# calculate variance using a column-wise operation with function apply
mort_var <- apply(mort_values, MARGIN = 2, FUN = var, na.rm = TRUE)

# same for HDI
hdi_values <- hdi_countries[,2:ncol(hdi_countries)]
hdi_var <- apply(hdi_values, MARGIN = 2, FUN = var, na.rm = TRUE)

# set the plot window into a 2x2 grid - can be reset by clearing plots or by
# par(mfrow = c(1, 1))
par(mfrow = c(2, 2))

# timesteps in which there is data
mort_ts <- 1940:2015

# plot
plot(mort_ts, mort_var, pch = 20, col = "darkblue", # x, y, shape, color
     main = "Under 5-year mortality - Variance between countries", # title
     xlab = "Year", ylab = "Variance [-]") # no ticks to x, labels

# timesteps in which there is data
hdi_ts <- c(1980, 1990, 2000, 2005, 2006, 2007, 2008, 2009, 2011)

# plot
plot(hdi_ts, hdi_var, pch = 20, col = "orange",
     main = "HDI - Variance between countries",
     xlab = "Year", ylab = "Variance [-]")

# TODO: calculate variance for life expectancy and GDP
expect_var <- NA # REPLACE NA WITH YOUR OWN CODE HERE
gdp_var <- NA # REPLACE NA WITH YOUR OWN CODE HERE
  
# TODO: add plots of annual variance for life expectancy and GDP to the figure
# with under five-year mortality and HDI variance

# WRITE YOUR OWN CODE HERE
  
par(mfrow = c(1, 1))
  
# Calculate and plot coefficient of variation (CV) -----------------------------

# Calculate CV for each year. First, calculate mean and standard deviation for
# each year using apply function, then calculate CV (cv = (sd / mean) * 100)
mort_mean <- apply(mort_values, MARGIN = 2, FUN = mean, na.rm = TRUE)
mort_sd <- apply(mort_values, MARGIN = 2, FUN = sd, na.rm = TRUE)
mort_cv <- (mort_sd / mort_mean) * 100

hdi_mean <- apply(hdi_values, MARGIN = 2, FUN = mean, na.rm = TRUE)
hdi_sd <- apply(hdi_values, MARGIN = 2, FUN = sd, na.rm = TRUE)
hdi_cv <- (hdi_sd / hdi_mean) * 100

# plot CV
par(mfrow = c(2, 2))
plot(mort_ts, mort_cv, pch = 20, col = "darkblue",
     main = "Under 5-year mortality - CV between countries",
     xlab = "Year", ylab = "CV [%]")

plot(hdi_ts, hdi_cv, pch = 20, col = "orange",
     main = "HDI - CV between countries",
     xlab = "Year", ylab = "CV [%]")

# TODO: calculate CV for life expectancy and GDP

# WRITE YOUR OWN CODE HERE
  
# TODO: add plots of annual variance for life expectancy and GDP to the figure
# with under five-year mortality and HDI variance

# WRITE YOUR OWN CODE HERE

par(mfrow = c(1, 1))

################################################################################
# End of guided exercise -------------------------------------------------------
#   Examples of using tidyverse, a versatile data manipulation ecosystem, will
#   follow. The learning curve to using tidyverse is rather steep at the
#   beginning but it is extremely powerful in use. The following parts are not
#   required for exercise completion but worth a look if you'd like to learn.


################################################################################
# tidyverse examples -----------------------------------------------------------
#   Not required for exercise completion.


library(tidyverse)

# test of converting data from wide to long format
mort_long <- under_five_mort_countries %>%
  rename(country = "Under.five.mortality") %>%
  pivot_longer(cols = -country, names_to = "year") %>%
  mutate(year = str_replace(year, "X", "") %>% as.numeric()) %>%
  mutate(variable = "Under five-year mortality")

# filter out e.g. Finland, Sweden, and Norway
finswenor <- mort_long %>%
  filter(country %in% c("Finland", "Sweden", "Norway"))

# now e.g. plotting becomes a bit less tedious
ggplot(data = finswenor) +
  geom_line(aes(x = year, y = value, color = country))

# do the same conversion to long format for all data
exp_long <- life_exp_countries %>%
  rename(country = "Life.expectancy.with.projections") %>%
  pivot_longer(cols = -country, names_to = "year") %>%
  mutate(year = str_replace(year, "X", "") %>% as.numeric()) %>%
  mutate(variable = "Life expectancy")

hdi_long <- hdi_countries %>%
  rename(country = "HDI") %>%
  pivot_longer(cols = -country, names_to = "year") %>%
  mutate(year = str_replace(year, "X", "") %>% as.numeric()) %>%
  mutate(variable = "Human Development Index")

gdp_long <- gdp_countries %>%
  rename(country = "GDP.per.capita") %>%
  pivot_longer(cols = -country, names_to = "year") %>%
  mutate(year = str_replace(year, "X", "") %>% as.numeric()) %>%
  mutate(variable = "Gross Domestic Product")

# bind all data together (results in 61620 x 4 tibble)
all_data <- mort_long %>%
  bind_rows(exp_long, hdi_long, gdp_long)

# With the data in long format, we can calculate all variances and CVs at once.
# For each variable and each year, summarise the variance (across countries).
all <- all_data %>%
  group_by(variable, year) %>%
  mutate(variance = var(value, na.rm = TRUE)) %>%
  mutate(mean_for_cv = mean(value, na.rm = TRUE)) %>%
  mutate(sd_for_cv = sd(value, na.rm = TRUE)) %>%
  mutate(cv = (sd_for_cv / mean_for_cv) * 100) %>%
  select(-c(mean_for_cv, sd_for_cv))

# see now e.g. year 1990 for Finland, Sweden, and Norway - grouping goes correct
all %>% filter(year == 1990 & country %in% c("Finland", "Sweden", "Norway"))

# then we can plot again using facets for each variable
ggplot(data = all) +
  geom_point(aes(x = year, y = variance, color = variable)) +
  facet_wrap(~variable, scales = "free") +
  ylab("Variance [-]") +
  ggtitle("Variance between countries")

# same, but for CV - change only the y variable in the ggplot call
ggplot(data = all) +
  geom_point(aes(x = year, y = cv, color = variable)) +
  facet_wrap(~variable, scales = "free") +
  ylab("CV [%]") +
  ggtitle("Coefficient of variation (CV) between countries")

